Module: Origen
- Defined in:
- lib/test_ids/origen/origen.rb
Class Method Summary collapse
-
._orig_reset_interface ⇒ Object
Override the Origen.reset_interface method to clear out the TestIds configuration, so that it doesn't carry over from one flow to the next.
- .reset_interface(options = {}) ⇒ Object
Class Method Details
._orig_reset_interface ⇒ Object
Override the Origen.reset_interface method to clear out the TestIds configuration, so that it doesn't carry over from one flow to the next
8 |
# File 'lib/test_ids/origen/origen.rb', line 8 alias_method :_orig_reset_interface, :reset_interface |
.reset_interface(options = {}) ⇒ Object
9 10 11 12 |
# File 'lib/test_ids/origen/origen.rb', line 9 def reset_interface( = {}) TestIds.send(:clear_configuration_id) _orig_reset_interface() end |