Module: OrigenTesters::Flow
- Defined in:
- lib/test_ids/origen_testers/flow.rb
Constant Summary collapse
- BIN_OPTS =
%i(bin softbin bin_size softbin_size number number_size).freeze
Instance Method Summary collapse
-
#_orig_test ⇒ Object
Override the flow.test method to inject our generated bin and test numbers.
- #test(instance, options = {}) ⇒ Object
Instance Method Details
#_orig_test ⇒ Object
Override the flow.test method to inject our generated bin and test numbers
10 |
# File 'lib/test_ids/origen_testers/flow.rb', line 10 alias_method :_orig_test, :test |
#test(instance, options = {}) ⇒ Object
11 12 13 14 15 16 17 18 19 20 21 22 23 24 |
# File 'lib/test_ids/origen_testers/flow.rb', line 11 def test(instance, = {}) if TestIds.configured? && [:test_ids] != :notrack [:test_ids_flow_id] = try(:top_level).try(:id) || id TestIds.current_configuration.allocator.allocate(instance, ) unless TestIds.current_configuration.send_to_ate? BIN_OPTS.each do |opt| .delete(opt) end end end _orig_test(instance, ) end |